@@ -116,7 +116,7 @@ create table wkf
 (
     id serial,
     name varchar,
-    osv varchar,
+    model varchar,
     on_create bool,
     primary key(id)
 );
@@ -126,8 +126,8 @@ create table wkf_activity
     id serial,
     workflow int references wkf on delete cascade,
     subflow int references wkf on delete set null,
-    split_mode varchar(3),
-    join_mode varchar(3),
+    split_mode varchar,
+    join_mode varchar,
     kind varchar not null,
     name varchar,
     signal_send varchar,
@@ -183,8 +183,8 @@ create table wkf_witm_trans
 
 CREATE TABLE ir_module_module (
     id serial NOT NULL,
-    create_uid integer references res_user on delete set null,
-    create_date timestamp without time zone,
+    create_uid integer NOT NULL references res_user on delete set null,
+    create_date timestamp without time zone NOT NULL,
     write_date timestamp without time zone,
     write_uid integer references res_user on delete set null,
     website varchar,
@@ -200,8 +200,8 @@ ALTER TABLE ir_module_module add constraint name_uniq unique (name);
 
 CREATE TABLE ir_module_module_dependency (
     id serial NOT NULL,
-    create_uid integer references res_user on delete set null,
-    create_date timestamp without time zone,
+    create_uid integer NOT NULL references res_user on delete set null,
+    create_date timestamp without time zone NOT NULL,
     write_date timestamp without time zone,
     write_uid integer references res_user on delete set null,
     name varchar,
