@@ -64,10 +64,22 @@ class TableHandler(TableHandlerInterface):
 
     @staticmethod
     def table_rename(cursor, old_name, new_name):
+        #Rename table
         if TableHandler.table_exist(cursor, old_name) and \
                 not TableHandler.table_exist(cursor, new_name):
             cursor.execute('ALTER TABLE "%s" RENAME TO "%s"' % \
                     (old_name, new_name))
+        # Rename sequence
+        old_sequence = old_name + '_id_seq'
+        new_sequence = new_name + '_id_seq'
+        TableHandler.sequence_rename(cursor, old_sequence, new_sequence)
+        #Rename history table
+        old_history = old_name + "__history"
+        new_history = new_name + "__history"
+        if TableHandler.table_exist(cursor, old_history) and \
+                not TableHandler.table_exist(cursor, new_history):
+            cursor.execute('ALTER TABLE "%s" RENAME TO "%s"' % \
+                    (old_history, new_history))
 
     @staticmethod
     def sequence_exist(cursor, sequence_name):
@@ -85,6 +97,18 @@ class TableHandler(TableHandlerInterface):
     def column_exist(self, column_name):
         return column_name in self._columns
 
+    def column_rename(self, old_name, new_name, exception=False):
+        if self.column_exist(old_name) and \
+                not self.column_exist(new_name):
+            self.cursor.execute('ALTER TABLE "%s" ' \
+                    'RENAME COLUMN "%s" TO "%s"' %
+                    (self.table_name, old_name, new_name))
+        elif exception and self.column_exist(new_name):
+            raise Exception('Unable to rename column %s.%s to %s.%s: ' \
+                    '%s.%s already exist!' % \
+                    (self.table_name, old_name, self.table_name, new_name,
+                     self.table_name, new_name))
+
     def _update_definitions(self):
         # Fetch columns definitions from the table
         self.cursor.execute("SELECT at.attname, at.attlen, "\
@@ -124,9 +148,8 @@ class TableHandler(TableHandlerInterface):
             contype, confdeltype, column, ref, conname = line
             if contype == 'f':
                 self._fk_deltypes[column] = confdeltype
-            else:
-                if conname not in self._constraints:
-                    self._constraints.append(conname)
+            if conname not in self._constraints:
+                self._constraints.append(conname)
 
         # Fetch indexes defined for the table
         self.cursor.execute("SELECT cl2.relname "\
@@ -260,9 +283,7 @@ class TableHandler(TableHandlerInterface):
         if not self.cursor.rowcount:
             add = True
         elif self._fk_deltypes.get(column_name) != code:
-            self.cursor.execute('ALTER TABLE "' + self.table_name + '" ' \
-                    'DROP CONSTRAINT "' + self.table_name + '_' + \
-                    column_name + '_fkey"')
+            self.drop_fk(column_name)
             add = True
         if add:
             self.cursor.execute('ALTER TABLE "' + self.table_name + '" ' \
@@ -271,10 +292,14 @@ class TableHandler(TableHandlerInterface):
                         'ON DELETE ' + on_delete)
         self._update_definitions()
 
-    def index_action(self, column_name, action='add'):
+    def drop_fk(self, column_name, table=None):
+        self.drop_constraint(column_name + '_fkey', table=table)
+
+    def index_action(self, column_name, action='add', table=None):
         if isinstance(column_name, basestring):
             column_name = [column_name]
-        index_name = self.table_name + "_" + '_'.join(column_name) + "_index"
+        index_name = (table or self.table_name) + "_" + '_'.join(column_name) \
+                + "_index"
 
         if action == 'add':
             if index_name in self._indexes:
@@ -286,7 +311,8 @@ class TableHandler(TableHandlerInterface):
             self._update_definitions()
         elif action == 'remove':
             if len(column_name) == 1:
-                if self._field2module.get(column_name[0]) != self.module_name:
+                if self._field2module.get(column_name[0], self.module_name) \
+                        != self.module_name:
                     return
 
             if index_name in self._indexes:
@@ -322,7 +348,8 @@ class TableHandler(TableHandlerInterface):
         elif action == 'remove':
             if not self._columns[column_name]['notnull']:
                 return
-            if self._field2module.get(column_name) != self.module_name:
+            if self._field2module.get(column_name, self.module_name) \
+                    != self.module_name:
                 return
             self.cursor.execute('ALTER TABLE "%s" ' \
                                'ALTER COLUMN "%s" DROP NOT NULL' %
@@ -352,8 +379,8 @@ class TableHandler(TableHandlerInterface):
                  ident, constraint,))
         self._update_definitions()
 
-    def drop_constraint(self, ident, exception=False):
-        ident = self.table_name + "_" + ident
+    def drop_constraint(self, ident, exception=False, table=None):
+        ident = (table or self.table_name) + "_" + ident
         if ident not in self._constraints:
             return
         try:
