@@ -1,4 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 import ConfigParser, optparse, os, sys
 from version import VERSION
 
@@ -13,11 +14,13 @@ class ConfigManager(object):
             'xmlport': 8069,
             'webdav': False,
             'webdavport': 8080,
+            'db_type': 'postgresql',
             'db_host': False,
             'db_port': False,
             'db_name': False,
             'db_user': False,
             'db_password': False,
+            'db_minconn': 1,
             'db_maxconn': 64,
             'pg_path': None,
             'admin_passwd': 'admin',
@@ -31,6 +34,9 @@ class ConfigManager(object):
             'privatekey': '/etc/ssl/trytond/server.key',
             'certificate': '/etc/ssl/trytond/server.pem',
             'smtp_server': 'localhost',
+            'smtp_port': 25,
+            'smtp_ssl': False,
+            'smtp_tls': False,
             'smtp_user': False,
             'smtp_password': False,
             'data_path': '/var/lib/trytond',
@@ -38,8 +44,12 @@ class ConfigManager(object):
             'multi_server': False,
             'session_timeout': 600,
             'psyco': False,
+            'auto_reload': True,
+            'init': {},
+            'update': {},
         }
 
+    def parse(self):
         parser = optparse.OptionParser(version=VERSION)
 
         parser.add_option("-c", "--config", dest="config",
