@@ -1,8 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Action"
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.tools import file_open
+from trytond.model import ModelView, ModelSQL, ModelStorage, fields
+from trytond.tools import file_open, Cache
 from trytond.backend import TableHandler
 import base64
 import os
@@ -204,7 +204,6 @@ ActionKeyword()
 class ActionReport(ModelSQL, ModelView):
     "Action report"
     _name = 'ir.action.report'
-    _sequence = 'ir_action_id_seq'
     _description = __doc__
     _inherits = {'ir.action': 'action'}
     model = fields.Char('Model')
@@ -225,7 +224,7 @@ class ActionReport(ModelSQL, ModelView):
         [('odt', 'ODT Document'),
          ('pdf', 'PDF Document'),],
         string='Extension', required=True)
-    module = fields.Char('Module', readonly=True)
+    module = fields.Char('Module', readonly=True, select=1)
     email = fields.Char('Email')
 
     def __init__(self):
@@ -327,6 +326,27 @@ class ActionReport(ModelSQL, ModelView):
             return new_ids[0]
         return new_ids
 
+    def create(self, cursor, user, vals, context=None):
+        later = {}
+        vals = vals.copy()
+        for field in vals:
+            if field in self._columns \
+                    and hasattr(self._columns[field], 'set'):
+                later[field] = vals[field]
+        for field in later:
+            del vals[field]
+        if cursor.nextid(self._table):
+            cursor.setnextid(self._table, cursor.currid('ir_action'))
+        new_id = super(ActionReport, self).create(cursor, user, vals,
+                context=context)
+        report = self.browse(cursor, user, new_id, context=context)
+        new_id = report.action.id
+        cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
+                'WHERE id = %s', (report.action.id, report.id))
+        ModelStorage.delete(self, cursor, user, report.id)
+        self.write(cursor, user, new_id, later, context=context)
+        return new_id
+
     def write(self, cursor, user, ids, vals, context=None):
         if context is None:
             context = {}
@@ -357,7 +377,6 @@ ActionReport()
 class ActionActWindow(ModelSQL, ModelView):
     "Action act window"
     _name = 'ir.action.act_window'
-    _sequence = 'ir_action_id_seq'
     _description = __doc__
     _inherits = {'ir.action': 'action'}
     domain = fields.Char('Domain Value')
@@ -378,7 +397,7 @@ class ActionActWindow(ModelSQL, ModelView):
             help='Add an auto-refresh on the view')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
-    window_name = fields.Boolean('Window Name', required=True,
+    window_name = fields.Boolean('Window Name',
             help='Use the action name as window name')
     search_value = fields.Char('Search Criteria',
             help='Default search criteria for the list view')
@@ -411,6 +430,27 @@ class ActionActWindow(ModelSQL, ModelView):
                     for view in act.act_window_views]
         return res
 
+    def create(self, cursor, user, vals, context=None):
+        later = {}
+        vals = vals.copy()
+        for field in vals:
+            if field in self._columns \
+                    and hasattr(self._columns[field], 'set'):
+                later[field] = vals[field]
+        for field in later:
+            del vals[field]
+        if cursor.nextid(self._table):
+            cursor.setnextid(self._table, cursor.currid('ir_action'))
+        new_id = super(ActionActWindow, self).create(cursor, user, vals,
+                context=context)
+        act_window = self.browse(cursor, user, new_id, context=context)
+        new_id = act_window.action.id
+        cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
+                'WHERE id = %s', (act_window.action.id, act_window.id))
+        ModelStorage.delete(self, cursor, user, act_window.id)
+        self.write(cursor, user, new_id, later, context=context)
+        return new_id
+
     def delete(self, cursor, user, ids, context=None):
         action_obj = self.pool.get('ir.action')
 
@@ -455,7 +495,6 @@ ActionActWindowView()
 class ActionWizard(ModelSQL, ModelView):
     "Action wizard"
     _name = 'ir.action.wizard'
-    _sequence = 'ir_action_id_seq'
     _description = __doc__
     _inherits = {'ir.action': 'action'}
     wiz_name = fields.Char('Wizard name', required=True)
@@ -463,10 +502,32 @@ class ActionWizard(ModelSQL, ModelView):
             ondelete='CASCADE')
     model = fields.Char('Model')
     email = fields.Char('Email')
+    window = fields.Boolean('Window', help='Run wizard in a new window')
 
     def default_type(self, cursor, user, context=None):
         return 'ir.action.wizard'
 
+    def create(self, cursor, user, vals, context=None):
+        later = {}
+        vals = vals.copy()
+        for field in vals:
+            if field in self._columns \
+                    and hasattr(self._columns[field], 'set'):
+                later[field] = vals[field]
+        for field in later:
+            del vals[field]
+        if cursor.nextid(self._table):
+            cursor.setnextid(self._table, cursor.currid('ir_action'))
+        new_id = super(ActionWizard, self).create(cursor, user, vals,
+                context=context)
+        wizard = self.browse(cursor, user, new_id, context=context)
+        new_id = wizard.action.id
+        cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
+                'WHERE id = %s', (wizard.action.id, wizard.id))
+        ModelStorage.delete(self, cursor, user, wizard.id)
+        self.write(cursor, user, new_id, later, context=context)
+        return new_id
+
     def delete(self, cursor, user, ids, context=None):
         action_obj = self.pool.get('ir.action')
 
@@ -483,10 +544,103 @@ class ActionWizard(ModelSQL, ModelView):
 ActionWizard()
 
 
+class ActionWizardSize(ModelSQL, ModelView):
+    "Action Wizard Size"
+    _name = 'ir.action.wizard_size'
+    _description = __doc__
+    wizard = fields.Char('Wizard', required=True, select=1)
+    model = fields.Char('Model', required=True, select=1)
+    user = fields.Many2One('res.user', 'User', required=True,
+            ondelete='CASCADE', select=1)
+    width = fields.Integer('Width')
+    height = fields.Integer('Height')
+
+    def __init__(self):
+        super(ActionWizardSize, self).__init__()
+        self._rpc.update({
+            'set_size': True,
+        })
+
+    def create(self, cursor, user, vals, context=None):
+        res = super(ActionWizardSize, self).create(cursor, user, vals,
+                context=context)
+        # Restart the cache for get_size
+        self.get_size(cursor.dbname)
+        return res
+
+    def write(self, cursor, user, ids, vals, context=None):
+        res = super(ActionWizardSize, self).write(cursor, user, ids,
+                vals, context=context)
+        # Restart the cache for get_size
+        self.get_size(cursor.dbname)
+        return res
+
+    def delete(self, cursor, user, ids, context=None):
+        res = super(ActionWizardSize, self).delete(cursor, user, ids,
+                context=context)
+        # Restart the cache for get_size
+        self.get_size(cursor.dbname)
+        return res
+
+    def set_size(self, cursor, user, wizard, model, width, height,
+            context=None):
+        '''
+        Set size for wizard dialog.
+        :param cursor: the database cursor
+        :param user: the user id
+        :param wizard: the wizard name
+        :param model: the model name
+        :param width: the width
+        :param height: the height
+        :param context: the context
+        '''
+        ids = self.search(cursor, user, [
+            ('user', '=', user),
+            ('wizard', '=', wizard),
+            ('model', '=', model),
+            ], context=context)
+        if ids:
+            self.write(cursor, user, ids, {
+                'width': width,
+                'height': height,
+                }, context=context)
+        else:
+            self.create(cursor, user, {
+                'wizard': wizard,
+                'model': model,
+                'user': user,
+                'width': width,
+                'height': height,
+                }, context=context)
+
+    def get_size(self, cursor, user, wizard, model, context=None):
+        '''
+        Get size for wizard dialog.
+        :param cursor: the database cursor
+        :param user: the user id
+        :param wizard: the wizard name
+        :param model: the model name
+        :param context: the context
+        :return: (width, height)
+        '''
+        ids = self.search(cursor, user, [
+            ('user', '=', user),
+            ('wizard', '=', wizard),
+            ('model', '=', model),
+            ], limit=1, context=context)
+        if ids:
+            wizard_size = self.browse(cursor, user, ids[0], context=context)
+            return wizard_size.width, wizard_size.height
+        return (0, 0)
+
+    get_size = Cache('ir.action.wizard_size.get_size')(get_size)
+
+ActionWizardSize()
+
+
 class ActionURL(ModelSQL, ModelView):
     "Action URL"
     _name = 'ir.action.url'
-    _sequence = 'ir_action_id_seq'
     _description = __doc__
     _inherits = {'ir.action': 'action'}
     url = fields.Char('Action Url', required=True)
@@ -499,6 +653,27 @@ class ActionURL(ModelSQL, ModelView):
     def default_target(self, cursor, user, context=None):
         return 'new'
 
+    def create(self, cursor, user, vals, context=None):
+        later = {}
+        vals = vals.copy()
+        for field in vals:
+            if field in self._columns \
+                    and hasattr(self._columns[field], 'set'):
+                later[field] = vals[field]
+        for field in later:
+            del vals[field]
+        if cursor.nextid(self._table):
+            cursor.setnextid(self._table, cursor.currid('ir_action'))
+        new_id = super(ActionURL, self).create(cursor, user, vals,
+                context=context)
+        url = self.browse(cursor, user, new_id, context=context)
+        new_id = url.action.id
+        cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
+                'WHERE id = %s', (url.action.id, url.id))
+        ModelStorage.delete(self, cursor, user, url.id)
+        self.write(cursor, user, new_id, later, context=context)
+        return new_id
+
     def delete(self, cursor, user, ids, context=None):
         action_obj = self.pool.get('ir.action')
 
