@@ -1,14 +1,18 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 "Lang"
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.model.cacheable import Cacheable
+from trytond.tools import safe_eval
 import time
+from time_locale import TIME_LOCALE
 
 
-class Lang(ModelSQL, ModelView):
+class Lang(ModelSQL, ModelView, Cacheable):
     "Language"
     _name = "ir.lang"
     _description = __doc__
-    name = fields.Char('Name', required=True)
+    name = fields.Char('Name', required=True, translate=True)
     code = fields.Char('Code', required=True,
             help="RFC 4646 tag: http://tools.ietf.org/html/rfc4646")
     translatable = fields.Boolean('Translatable')
@@ -42,6 +46,33 @@ class Lang(ModelSQL, ModelView):
             'invalid_date': 'The date format is not valid!',
         })
 
+    def read(self, cursor, user, ids, fields_names=None, context=None):
+        translation_obj = self.pool.get('ir.translation')
+        if context is None:
+            context = {}
+        res = super(Lang, self).read(cursor, user, ids,
+                fields_names=fields_names, context=context)
+        if context.get('translate_name', False) \
+                and (not fields_names or 'name' in fields_names):
+            ctx = context.copy()
+            ctx['language'] = self.default_code(cursor, user, context=context)
+            del ctx['translate_name']
+            res2 = self.read(cursor, user, ids,
+                    fields_names=['id', 'code', 'name'], context=ctx)
+            for record2 in res2:
+                for record in res:
+                    if record['id'] == record2['id']:
+                        break
+                res_trans = translation_obj._get_ids(cursor,
+                        self._name + ',name', 'model',
+                        record2['code'], [record2['id']])
+                record['name'] = res_trans.get(record2['id'], False) \
+                        or record2['name']
+        return res
+
+    def default_code(self, cursor, user, context=None):
+        return 'en_US'
+
     def default_active(self, cursor, user, context=None):
         return 1
 
@@ -69,7 +100,7 @@ class Lang(ModelSQL, ModelView):
         '''
         for lang in self.browse(cursor, user, ids):
             try:
-                grouping = eval(lang.grouping)
+                grouping = safe_eval(lang.grouping)
                 for i in grouping:
                     if not isinstance(i, int):
                         return False
@@ -98,19 +129,56 @@ class Lang(ModelSQL, ModelView):
                     and '%j' not in lang.date \
                     and '%-j' not in lang.date:
                 return False
+            if '%x' in lang.date \
+                    or '%X' in lang.date \
+                    or '%c' in lang.date \
+                    or '%Z' in lang.date:
+                return False
         return True
 
     def check_xml_record(self, cursor, user, ids, values, context=None):
         return True
 
+    def get_translatable_languages(self, cursor, user, context=None):
+        res = self.get(cursor, 'translatable_languages')
+        if res is None:
+            lang_ids = self.search(cursor, user, [
+                    ('translatable', '=', True),
+                    ], context=context)
+            res = [x.code for x in self.browse(cursor, user, lang_ids,
+                context=context)]
+            self.add(cursor, 'translatable_languages', res)
+        return res
+
+    def create(self, cursor, user, vals, context=None):
+        # Clear cache
+        if self.get(cursor, 'translatable_languages'):
+            self.invalidate(cursor, 'translatable_languages')
+        return super(Lang, self).create(cursor, user, vals,
+                     context=context)
+
+    def write(self, cursor, user, ids, vals, context=None):
+        # Clear cache
+        if self.get(cursor, 'translatable_languages'):
+            self.invalidate(cursor, 'translatable_languages')
+        return super(Lang, self).write(cursor, user, ids, vals,
+                     context=context)
+
+    def delete(self, cursor, user, ids, context=None):
+        # Clear cache
+        if self.get(cursor, 'translatable_languages'):
+            self.invalidate(cursor, 'translatable_languages')
+        return super(Lang, self).delete(cursor, user, ids,
+                     context=context)
+
     def _group(self, lang, s, monetary=False):
         # Code from _group in locale.py
         if monetary:
             thousands_sep = monetary['mon_thousands_sep']
-            grouping = eval(monetary['mon_grouping'])
+            grouping = safe_eval(monetary['mon_grouping'])
         else:
             thousands_sep = lang['thousands_sep']
-            grouping = eval(lang['grouping'])
+            grouping = safe_eval(lang['grouping'])
         if not grouping:
             return (s, 0)
         result = ""
@@ -258,4 +326,20 @@ class Lang(ModelSQL, ModelView):
 
         return s.replace('<', '').replace('>', '')
 
+    def strftime(self, timetuple, code, format):
+        '''
+        Convert timetuple to a string as specified by the format argument.
+
+        :param timetuple: a tuple or struct_time representing a time
+        :param code: locale code
+        :param format: a string
+        :return: a unicode string
+        '''
+        if code in TIME_LOCALE:
+            for f, i in (('%a', 6), ('%A', 6), ('%b', 1), ('%B', 1)):
+                format = format.replace(f, TIME_LOCALE[code][f][timetuple[i]])
+            format = format.replace('%p', TIME_LOCALE[code]['%p']\
+                    [timetuple[3] < 12 and 0 or 1]).encode('utf-8')
+        return time.strftime(format, timetuple).decode('utf-8')
+
 Lang()
