@@ -95,7 +95,7 @@ class ModelField(ModelSQL, ModelView):
                     model_ids.add(rec['model'])
             model_ids = list(model_ids)
             cursor.execute('SELECT id, model FROM ir_model WHERE id IN ' \
-                    '(' + ','.join(['%s' for x in model_ids]) + ')', model_ids)
+                    '(' + ','.join(('%s',) * len(model_ids)) + ')', model_ids)
             id2model = dict(cursor.fetchall())
 
             trans_args = []
@@ -194,12 +194,12 @@ class ModelAccess(ModelSQL, ModelView):
             return True
         ir_model_obj = self.pool.get('ir.model')
         user_group_obj = self.pool.get('res.user-res.group')
-        cursor.execute('SELECT MAX(CASE WHEN perm_'+mode+' THEN 1 else 0 END) '
+        cursor.execute('SELECT MAX(CASE WHEN a.perm_'+mode+' THEN 1 else 0 END) '
             'FROM ir_model_access a '
                 'JOIN "' + ir_model_obj._table + '" m '
                     'ON (a.model = m.id) '
                 'JOIN "' + user_group_obj._table + '" gu '
-                    'ON (gu.gid = a.group) '
+                    'ON (gu.gid = a."group") '
             'WHERE m.model = %s AND gu.uid = %s', (model_name, user,))
         row = cursor.fetchall()
         if row[0][0] is None:
@@ -208,7 +208,7 @@ class ModelAccess(ModelSQL, ModelView):
                     'FROM ir_model_access a ' \
                     'JOIN ir_model m ' \
                         'ON (a.model = m.id) ' \
-                    'WHERE a.group IS NULL AND m.model = %s', (model_name,))
+                    'WHERE a."group" IS NULL AND m.model = %s', (model_name,))
             row = cursor.fetchall()
             if row[0][0] is None:
                 return True
@@ -229,9 +229,9 @@ class ModelAccess(ModelSQL, ModelView):
                 context=context)
         self.check(cursor.dbname)
         # Restart the cache
-        for model in self.pool.object_name_list():
+        for _, model in self.pool.iterobject():
             try:
-                self.pool.get(model).fields_view_get(cursor.dbname)
+                model.fields_view_get(cursor.dbname)
             except:
                 pass
         return res
@@ -241,9 +241,9 @@ class ModelAccess(ModelSQL, ModelView):
                 context=context)
         self.check(cursor.dbname)
         # Restart the cache
-        for model in self.pool.object_name_list():
+        for _, model in self.pool.iterobject():
             try:
-                self.pool.get(model).fields_view_get(cursor.dbname)
+                model.fields_view_get(cursor.dbname)
             except:
                 pass
         return res
@@ -253,9 +253,9 @@ class ModelAccess(ModelSQL, ModelView):
                 context=context)
         self.check(cursor.dbname)
         # Restart the cache
-        for model in self.pool.object_name_list():
+        for _, model in self.pool.iterobject():
             try:
-                self.pool.get(model).fields_view_get(cursor.dbname)
+                model.fields_view_get(cursor.dbname)
             except:
                 pass
         return res
@@ -359,10 +359,19 @@ class ModelGraph(Report):
     def execute(self, cursor, user, ids, datas, context=None):
         import pydot
         model_obj = self.pool.get('ir.model')
+        action_report_obj = self.pool.get('ir.action.report')
 
         if context is None:
             context = {}
 
+        action_report_ids = action_report_obj.search(cursor, user, [
+            ('report_name', '=', self._name)
+            ], context=context)
+        if not action_report_ids:
+            raise Exception('Error', 'Report (%s) not find!' % self._name)
+        action_report = action_report_obj.browse(cursor, user,
+                action_report_ids[0], context=context)
+
         models = model_obj.browse(cursor, user, ids, context=context)
 
         graph = pydot.Dot(fontsize="8")
@@ -371,7 +380,7 @@ class ModelGraph(Report):
         self.fill_graph(cursor, user, models, graph,
                 level=datas['form']['level'], context=context)
         data = graph.create(prog='dot', format='png')
-        return ('png', base64.encodestring(data), False)
+        return ('png', base64.encodestring(data), False, action_report.name)
 
     def fill_graph(self, cursor, user, models, graph, level=1, context=None):
         import pydot
