@@ -158,7 +158,6 @@ class Module(ModelSQL, ModelView):
             'button_uninstall_cancel': True,
             'button_upgrade': True,
             'button_upgrade_cancel': True,
-            'button_update_translations': True,
             'on_write': False,
         })
         self._error_messages.update({
@@ -181,7 +180,7 @@ class Module(ModelSQL, ModelView):
             else:
                 file_p = tools.file_open(os.path.join(name, '__tryton__.py'))
             data = file_p.read()
-            info = eval(data)
+            info = tools.safe_eval(data)
             file_p.close()
         except:
             return {}
@@ -219,26 +218,24 @@ class Module(ModelSQL, ModelView):
             if module.name not in graph:
                 continue
             def get_parents(name, graph):
-                parents = []
+                parents = set()
                 for node in graph:
                     if graph[name] in node.childs:
-                        if node.name not in parents:
-                            parents.append(node.name)
-                for parent in parents:
-                    for parent2 in get_parents(parent, graph):
-                        if parent2 not in parents:
-                            parents.append(parent2)
+                        parents.add(node.name)
+                for parent in parents.copy():
+                    parents.update(get_parents(parent, graph))
                 return parents
             dependencies = get_parents(module.name, graph)
             def get_childs(name, graph):
-                childs = [x.name for x in graph[name].childs]
-                childs2 = []
+                childs = set(x.name for x in graph[name].childs)
+                childs2 = set()
                 for child in childs:
-                    childs2 += get_childs(child, graph)
-                return childs + childs2
-            dependencies += get_childs(module.name, graph)
+                    childs2.update(get_childs(child, graph))
+                childs.update(childs2)
+                return childs
+            dependencies.update(get_childs(module.name, graph))
             res += self.search(cursor, user, [
-                ('name', 'in', dependencies),
+                ('name', 'in', list(dependencies)),
                 ], context=context)
         return list({}.fromkeys(res))
 
@@ -253,17 +250,14 @@ class Module(ModelSQL, ModelView):
                 self.raise_user_error(cursor, 'missing_dep',
                         (missings, module.name), context=context)
             def get_parents(name, graph):
-                parents = []
+                parents = set()
                 for node in graph:
                     if graph[name] in node.childs:
-                        if node.name not in parents:
-                            parents.append(node.name)
-                for parent in parents:
-                    for parent2 in get_parents(parent, graph):
-                        if parent2 not in parents:
-                            parents.append(parent2)
+                        parents.add(node.name)
+                for parent in parents.copy():
+                    parents.update(get_parents(parent, graph))
                 return parents
-            dependencies = get_parents(module.name, graph)
+            dependencies = list(get_parents(module.name, graph))
             module_install_ids = self.search(cursor, user, [
                 ('name', 'in', dependencies),
                 ('state', '=', 'uninstalled'),
@@ -283,12 +277,13 @@ class Module(ModelSQL, ModelView):
                 self.raise_user_error(cursor, user, 'missing_dep',
                         (missings, module.name), context=context)
             def get_childs(name, graph):
-                childs = [x.name for x in graph[name].childs]
-                childs2 = []
+                childs = set(x.name for x in graph[name].childs)
+                childs2 = set()
                 for child in childs:
-                    childs2 += get_childs(child, graph)
-                return childs + childs2
-            dependencies = get_childs(module.name, graph)
+                    childs2.update(get_childs(child, graph))
+                childs.update(childs2)
+                return childs
+            dependencies = list(get_childs(module.name, graph))
             module_installed_ids = self.search(cursor, user, [
                 ('name', 'in', dependencies),
                 ('state', '=', 'installed'),
@@ -319,7 +314,7 @@ class Module(ModelSQL, ModelView):
             if res:
                 self.raise_user_error(cursor, 'uninstall_dep',
                         error_description='\n'.join(
-                            ['\t%s: %s' % (x[0], x[1]) for x in res]),
+                            '\t%s: %s' % (x[0], x[1]) for x in res),
                         context=context)
         self.write(cursor, user, ids, {'state': 'to remove'})
         return True
@@ -335,20 +330,6 @@ class Module(ModelSQL, ModelView):
         self.write(cursor, user, ids, {'state': 'installed'}, context=context)
         return True
 
-    def button_update_translations(self, cursor, user, ids, context=None):
-        lang_obj = self.pool.get('ir.lang')
-        lang_ids = lang_obj.search(cursor, user, [('translatable', '=', True)],
-                context=context)
-        langs = lang_obj.browse(cursor, user, lang_ids, context=context)
-        for module in self.browse(cursor, user, ids, context=context):
-            files = Module.get_module_info(module.name).get('translations', {})
-            for lang in langs:
-                if files.has_key(lang.code):
-                    filepath = os.path.join(MODULES_PATH, module.name,
-                            files[lang.code])
-                    tools.trans_load(filepath, lang.code)
-        return True
-
     # update the list of available packages
     def update_list(self, cursor, user, context=None):
         lang_obj = self.pool.get('ir.lang')
@@ -647,8 +628,8 @@ class ModuleInstallUpgrade(Wizard):
             ], context=context)
         modules = module_obj.browse(cursor, user, module_ids, context=context)
         return {
-            'module_info': '\n'.join([x.name + ': ' + x.state \
-                    for x in modules]),
+            'module_info': '\n'.join(x.name + ': ' + x.state \
+                    for x in modules),
         }
 
     def _upgrade_module(self, cursor, user, data, context):
