@@ -3,6 +3,7 @@
 import time
 from trytond.model import ModelView, ModelSQL, fields
 from string import Template
+import datetime
 
 
 class SequenceType(ModelSQL, ModelView):
@@ -117,8 +118,9 @@ class Sequence(ModelSQL, ModelView):
             cursor.execute('UPDATE "' + self._table + '" ' \
                     'SET number_next = number_next + number_increment, ' \
                         'write_uid = %s, ' \
-                        'write_date = NOW() ' \
-                    'WHERE id = %s AND active = True', (user, sequence.id,))
+                        'write_date = %s ' \
+                    'WHERE id = %s AND active = %s',
+                    (user, datetime.datetime.now(), sequence.id, True))
             if number_next:
                 return self._process(cursor, user, sequence.prefix, date=date,
                         context=context) + \
@@ -144,7 +146,8 @@ class SequenceStrict(Sequence):
     _description = __doc__
 
     def get_id(self, cursor, user, clause, context=None):
-        cursor.execute('LOCK TABLE "' + self._table + '"')
+        if cursor.has_lock():
+            cursor.execute('LOCK TABLE "' + self._table + '"')
         return super(SequenceStrict, self).get_id(cursor, user, clause,
                 context=context)
 
