@@ -1,4 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 "Translation"
 import base64
 try:
@@ -14,7 +15,7 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.model.cacheable import Cacheable
 from trytond.wizard import Wizard
 from trytond import tools
-from trytond.tools import file_open
+from trytond.tools import file_open, reduce_ids
 from trytond.backend import TableHandler
 import os
 
@@ -143,6 +144,7 @@ class Translation(ModelSQL, ModelView, Cacheable):
         if to_fetch:
             for i in range(0, len(to_fetch), cursor.IN_MAX):
                 sub_to_fetch = to_fetch[i:i + cursor.IN_MAX]
+                red_sql, red_ids = reduce_ids('res_id', sub_to_fetch)
                 cursor.execute('SELECT res_id, value ' \
                         'FROM ir_translation ' \
                         'WHERE lang = %s ' \
@@ -150,10 +152,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
                             'AND name = %s ' \
                             'AND value != \'\' ' \
                             'AND value IS NOT NULL ' \
-                            'AND fuzzy = false ' \
-                            'AND res_id in (' + \
-                                ','.join([str(x) for x in sub_to_fetch]) + ')',
-                        (lang, ttype, name))
+                            'AND fuzzy = %s ' \
+                            'AND ' + red_sql,
+                        [lang, ttype, name, False] + red_ids)
                 for res_id, value in cursor.fetchall():
                     self.add(cursor, (lang, ttype, name, res_id), value)
                     translations[res_id] = value
@@ -247,9 +248,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
                         'AND src = %s ' \
                         'AND value != \'\' ' \
                         'AND value IS NOT NULL ' \
-                        'AND fuzzy = false ' \
+                        'AND fuzzy = %s ' \
                         'AND res_id = 0',
-                    (lang, ttype, str(name), source))
+                    (lang, ttype, str(name), source, False))
         else:
             cursor.execute('SELECT value ' \
                     'FROM ir_translation ' \
@@ -258,9 +259,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
                         'AND name = %s ' \
                         'AND value != \'\' ' \
                         'AND value IS NOT NULL ' \
-                        'AND fuzzy = false ' \
+                        'AND fuzzy = %s ' \
                         'AND res_id = 0',
-                    (lang, ttype, str(name)))
+                    (lang, ttype, str(name), False))
         res = cursor.fetchone()
         if res:
             self.add(cursor, (lang, ttype, name, source), res[0])
@@ -301,24 +302,24 @@ class Translation(ModelSQL, ModelView, Cacheable):
                             'AND src = %s ' \
                             'AND value != \'\' ' \
                             'AND value IS NOT NULL ' \
-                            'AND fuzzy = false ' \
+                            'AND fuzzy = %s ' \
                             'AND res_id = 0)',
-                            (lang, ttype, str(name), source))]
+                            (lang, ttype, str(name), source, False))]
                 else:
                     clause += [('(lang = %s ' \
                             'AND type = %s ' \
                             'AND name = %s ' \
                             'AND value != \'\' ' \
                             'AND value IS NOT NULL ' \
-                            'AND fuzzy = false ' \
+                            'AND fuzzy = %s ' \
                             'AND res_id = 0)',
-                            (lang, ttype, str(name)))]
+                            (lang, ttype, str(name), False))]
         if clause:
             for i in range(0, len(clause), cursor.IN_MAX):
                 sub_clause = clause[i:i + cursor.IN_MAX]
                 cursor.execute('SELECT lang, type, name, src, value ' \
                         'FROM ir_translation ' \
-                        'WHERE ' + ' OR '.join([x[0] for x in sub_clause]),
+                        'WHERE ' + ' OR '.join(x[0] for x in sub_clause),
                         reduce(lambda x, y: x + y, [x[1] for x in sub_clause]))
                 for lang, ttype, name, source, value in cursor.fetchall():
                     if (name, ttype, lang, source) not in args:
@@ -347,9 +348,10 @@ class Translation(ModelSQL, ModelView, Cacheable):
                             'AND src = %s ',
                         (vals.get('name', ''), vals.get('res_id', 0), 'en_US',
                             vals.get('type', ''), vals.get('src', '')))
-                if cursor.rowcount:
+                fetchone = cursor.fetchone()
+                if fetchone:
                     vals = vals.copy()
-                    vals['module'] = cursor.fetchone()[0]
+                    vals['module'] = fetchone[0]
             else:
                 cursor.execute('SELECT module, src FROM ir_translation ' \
                         'WHERE name = %s ' \
@@ -358,9 +360,10 @@ class Translation(ModelSQL, ModelView, Cacheable):
                             'AND type = %s',
                         (vals.get('name', ''), vals.get('res_id', 0), 'en_US',
                             vals.get('type', '')))
-                if cursor.rowcount:
+                fetchone = cursor.fetchone()
+                if fetchone:
                     vals = vals.copy()
-                    vals['module'], vals['src'] = cursor.fetchone()
+                    vals['module'], vals['src'] = fetchone
         return super(Translation, self).create(cursor, user, vals,
                 context=context)
 
@@ -511,12 +514,8 @@ class Translation(ModelSQL, ModelView, Cacheable):
                         model = translation.name.split(',')[0]
                         if model in db_id2fs_id:
                             res_id = db_id2fs_id[model].get(res_id)
-                    try:
-                        val = int(res_id)
-                        if val != 0:
-                            continue
-                    except (ValueError, TypeError):
-                        pass
+                        else:
+                            break
                     row.append(res_id)
                 elif field == 'fuzzy':
                     row.append(int(translation[field]))
@@ -673,13 +672,13 @@ class ReportTranslationSet(Wizard):
                     if seqmatch.ratio() > 0.6:
                         cursor.execute('UPDATE ir_translation ' \
                                 'SET src = %s, ' \
-                                    'fuzzy = True ' \
+                                    'fuzzy = %s ' \
                                 'WHERE name = %s ' \
                                     'AND type = %s ' \
                                     'AND src = %s ' \
                                     'AND module = %s',
-                                (string, report.report_name, 'odt', string_trans,
-                                    report.module))
+                                (string, True, report.report_name, 'odt',
+                                    string_trans, report.module))
                         del trans_reports[string_trans]
                         done = True
                         break
@@ -695,7 +694,7 @@ class ReportTranslationSet(Wizard):
                             'AND type = %s ' \
                             'AND module = %s ' \
                             'AND src NOT IN ' \
-                                '(' + ','.join(['%s' for x in strings]) + ')',
+                                '(' + ','.join(('%s',) * len(strings)) + ')',
                         (report.report_name, 'odt', report.module) + \
                                 tuple(strings))
         return {}
@@ -888,6 +887,7 @@ class TranslationClean(Wizard):
                             'delete_workflow_record',
                             'domain_validation_record',
                             'required_validation_record',
+                            'size_validation_record',
                             'access_error',
                             'read_error',
                             'write_error',
@@ -1004,13 +1004,13 @@ class TranslationUpdate(Wizard):
                 (data['form']['lang'],))
         for row in cursor.dictfetchall():
             cursor.execute('UPDATE ir_translation ' \
-                    'SET fuzzy = True, ' \
+                    'SET fuzzy = %s, ' \
                         'src = %s ' \
                     'WHERE name = %s ' \
                         'AND res_id = %s ' \
                         'AND type = %s ' \
                         'AND lang = %s',
-                    (row['src'], row['name'], row['res_id'], row['type'],
+                    (True, row['src'], row['name'], row['res_id'], row['type'],
                         data['form']['lang']))
 
         cursor.execute('SELECT src, MAX(value) AS value FROM ir_translation ' \
@@ -1024,17 +1024,17 @@ class TranslationUpdate(Wizard):
 
         for row in cursor.dictfetchall():
             cursor.execute('UPDATE ir_translation ' \
-                    'SET fuzzy = True, ' \
+                    'SET fuzzy = %s, ' \
                         'value = %s ' \
                     'WHERE src = %s ' \
                         'AND (value = \'\' OR value IS NULL) ' \
-                        'AND lang = %s', (row['value'], row['src'],
+                        'AND lang = %s', (True, row['value'], row['src'],
                             data['form']['lang']))
 
         cursor.execute('UPDATE ir_translation ' \
-                'SET fuzzy = False ' \
+                'SET fuzzy = %s ' \
                 'WHERE (value = \'\' OR value IS NULL) ' \
-                    'AND lang = %s', (data['form']['lang'],))
+                    'AND lang = %s', (False, data['form']['lang'],))
         return {}
 
     def _action_translation_open(self, cursor, user, data, context):
