@@ -6,7 +6,7 @@ from trytond.model.fields.field import Field
 
 class Float(Field):
     '''
-    Define a float field (``float``)
+    Define a float field (``float``).
     '''
     _type = 'float'
 
@@ -15,8 +15,8 @@ class Float(Field):
             change_default=False, translate=False, select=0, on_change=None,
             on_change_with=None, depends=None, order_field=None, context=None):
         '''
-        :param digits: a list of two intergers which define the total
-            of digit and the number of decimal for the float.
+        :param digits: a list of two integers defining the total
+            of digits and the number of decimals of the float.
         '''
         super(Float, self).__init__(string=string, help=help, required=required,
                 readonly=readonly, domain=domain, states=states,
