@@ -6,7 +6,7 @@ from trytond.model.fields.field import Field
 
 class Many2One(Field):
     '''
-    Define many2one field (``int``)
+    Define many2one field (``int``).
     '''
     _type = 'many2one'
 
@@ -16,21 +16,21 @@ class Many2One(Field):
             change_default=False, translate=False, select=0, on_change=None,
             on_change_with=None, depends=None, order_field=None, context=None):
         '''
-        :param model_name: The name of the targeted model
+        :param model_name: The name of the targeted model.
         :param left: The name of the field to store the left value for
             Modified Preorder Tree Traversal.
             See http://en.wikipedia.org/wiki/Tree_traversal
-        :param right: The name ofthe field to store the right value. See left
+        :param right: The name of the field to store the right value. See left
         :param ondelete: Define the behavior of the record when the target
             record is deleted. (``CASCADE``, ``RESTRICT``, ``SET NULL``)
         :param datetime_field: The name of the field that contains the datetime
             value to read the target record.
         '''
         if datetime_field:
-            if ('depends' in args) and args['depends']:
-                args['depends'].append(datetime_field)
+            if depends:
+                depends.append(datetime_field)
             else:
-                args['depends'] = [datetime_field]
+                depends = [datetime_field]
         super(Many2One, self).__init__(string=string, help=help,
                 required=required, readonly=readonly, domain=domain,
                 states=states, priority=priority, change_default=change_default,
