@@ -10,10 +10,10 @@ class Model(object):
     """
     Define a model in Tryton.
 
-    :_name: a unique string to reference the model across all the platform.
-    :_inherits:
+    :_name: a unique string to reference the model throughout the platform.
+    :_inherits: a model from which this one inherits.
     :_description: a non technical description string.
-    :_rpc: a dictionary with method name allowed to be called remotely and
+    :_rpc: a dictionary with a method name allowed to be called remotely and
         a boolean as value that defines if it must be called in a commited
         transaction.
     :_error_messages: a dictionary mapping keywords to an error message. Eg.:
@@ -25,17 +25,19 @@ class Model(object):
     _description = ''
     pool = None #XXX change to avoid collision with field
     __columns = None
+    __xxx2many_targets = None
     __defaults = None
 
     def _reset_columns(self):
         self.__columns = None
+        self._reset_xxx2many_targets()
 
     def _getcolumns(self):
         if self.__columns:
             return self.__columns
         res = {}
         for attr in dir(self):
-            if attr in ('_columns', '_defaults'):
+            if attr in ('_columns', '_defaults', '_xxx2many_targets'):
                 continue
             if isinstance(getattr(self, attr), fields.Field):
                 res[attr] = getattr(self, attr)
@@ -118,15 +120,65 @@ class Model(object):
                 res[field_name] = (model, self._inherits[model],
                         self.pool.get(model)._inherit_fields[field_name][2])
         self._inherit_fields = res
+        self._reset_xxx2many_targets()
         # Update objects that uses this one to update their _inherits fields
         for obj_name in self.pool.object_name_list():
             obj = self.pool.get(obj_name)
             if self._name in obj._inherits:
                 obj._inherits_reload()
 
+    def _reset_xxx2many_targets(self):
+        self.__xxx2many_targets = None
+
+    def _getxxx2many_targets(self):
+        if self.__xxx2many_targets:
+            return self.__xxx2many_targets
+        to_cache = True
+
+        res = [(field_name, field.model_name)
+                for field_name, field in self._columns.iteritems()
+                if field._type == 'one2many']
+
+        for field_name, field in self._columns.iteritems():
+            if field._type != 'many2many':
+                continue
+            if hasattr(field, 'get_target'):
+                try:
+                    model_name = field.get_target(self.pool)._name
+                except KeyError:
+                    to_cache = False
+                    continue
+            else:
+                model_name = field.model_name
+            res.append((field_name, model_name))
+
+        res += [(field_name, field.model_name)
+                for _, (_, field_name, field) in \
+                        self._inherit_fields.iteritems()
+                if field._type == 'one2many']
+
+        for _, (_, field_name, field) in self._inherit_fields.iteritems():
+            if field._type != 'many2many':
+                continue
+            if hasattr(field, 'get_target'):
+                try:
+                    model_name = field.get_target(self.pool)._name
+                except KeyError:
+                    to_cache = False
+                    continue
+            else:
+                model_name = field.model_name
+            res.append((field_name, model_name))
+
+        if to_cache:
+            self.__xxx2many_targets = res
+        return res
+
+    _xxx2many_targets = property(fget=_getxxx2many_targets)
+
     def init(self, cursor, module_name):
         """
-        Add model in ir.model and ir.model.field
+        Add model in ir.model and ir.model.field.
 
         :param cursor: the database cursor
         :param module_name: the module name
@@ -135,7 +187,14 @@ class Model(object):
         # Add model in ir_model
         cursor.execute("SELECT id FROM ir_model WHERE model = %s",
                 (self._name,))
-        if not cursor.rowcount:
+        model_id = None
+        if cursor.rowcount == -1 or cursor.rowcount is None:
+            data = cursor.fetchone()
+            if data:
+                model_id, = data
+        elif cursor.rowcount != 0:
+            model_id, = cursor.fetchone()
+        if not model_id:
             cursor.execute("INSERT INTO ir_model " \
                     "(model, name, info, module) VALUES (%s, %s, %s, %s)",
                     (self._name, self._description, self.__doc__,
@@ -144,7 +203,6 @@ class Model(object):
                     (self._name,))
             (model_id,) = cursor.fetchone()
         else:
-            (model_id,) = cursor.fetchone()
             cursor.execute('UPDATE ir_model ' \
                     'SET name = %s, ' \
                         'info = %s ' \
@@ -159,13 +217,19 @@ class Model(object):
                         'AND name = %s ' \
                         'AND res_id = %s',
                     ('en_US', 'model', name, 0))
-            if not cursor.rowcount:
+            trans_id = None
+            if cursor.rowcount == -1 or cursor.rowcount is None:
+                data = cursor.fetchone()
+                if data:
+                    trans_id, = data
+            elif cursor.rowcount != 0:
+                trans_id, = cursor.fetchone()
+            if not trans_id:
                 cursor.execute('INSERT INTO ir_translation ' \
                         '(name, lang, type, src, value, module, fuzzy) ' \
-                        'VALUES (%s, %s, %s, %s, %s, %s, false)',
-                        (name, 'en_US', 'model', src, '', module_name))
+                        'VALUES (%s, %s, %s, %s, %s, %s, %s)',
+                        (name, 'en_US', 'model', src, '', module_name, False))
             else:
-                trans_id = cursor.fetchone()[0]
                 cursor.execute('UPDATE ir_translation ' \
                         'SET src = %s ' \
                         'WHERE id = %s',
@@ -190,7 +254,7 @@ class Model(object):
                     'WHERE lang = %s ' \
                         'AND type IN (%s, %s, %s) ' \
                         'AND name IN ' \
-                            '(' + ','.join(['%s' for x in self._columns]) + ')',
+                            '(' + ','.join(('%s',) * len(self._columns)) + ')',
                             ('en_US', 'field', 'help', 'selection') + \
                                     tuple([self._name + ',' + x \
                                         for x in self._columns]))
@@ -238,9 +302,9 @@ class Model(object):
                             'write_uid', 'write_date', 'id'):
                     cursor.execute('INSERT INTO ir_translation ' \
                             '(name, lang, type, src, value, module, fuzzy) ' \
-                            'VALUES (%s, %s, %s, %s, %s, %s, false)',
+                            'VALUES (%s, %s, %s, %s, %s, %s, %s)',
                             (trans_name, 'en_US', 'field',
-                                field.string, '', module_name))
+                                field.string, '', module_name, False))
             elif trans_fields[trans_name]['src'] != field.string:
                 cursor.execute('UPDATE ir_translation ' \
                         'SET src = %s ' \
@@ -250,9 +314,9 @@ class Model(object):
                 if field.help:
                     cursor.execute('INSERT INTO ir_translation ' \
                             '(name, lang, type, src, value, module, fuzzy) ' \
-                            'VALUES (%s, %s, %s, %s, %s, %s, false)',
+                            'VALUES (%s, %s, %s, %s, %s, %s, %s)',
                             (trans_name, 'en_US', 'help',
-                                field.help, '', module_name))
+                                field.help, '', module_name, False))
             elif trans_help[trans_name]['src'] != field.help:
                 cursor.execute('UPDATE ir_translation ' \
                         'SET src = %s ' \
@@ -269,9 +333,9 @@ class Model(object):
                         cursor.execute('INSERT INTO ir_translation ' \
                                 '(name, lang, type, src, value, ' \
                                     'module, fuzzy) ' \
-                                'VALUES (%s, %s, %s, %s, %s, %s, false)',
+                                'VALUES (%s, %s, %s, %s, %s, %s, %s)',
                                 (trans_name, 'en_US', 'selection', val, '',
-                                    module_name))
+                                    module_name, False))
         # Clean ir_model_field from field that are no more existing.
         for field_name in fields:
             if fields[field_name]['module'] == module_name \
@@ -296,8 +360,9 @@ class Model(object):
             if error not in trans_error:
                 cursor.execute('INSERT INTO ir_translation ' \
                         '(name, lang, type, src, value, module, fuzzy) ' \
-                        'VALUES (%s, %s, %s, %s, %s, %s, false)',
-                        (self._name, 'en_US', 'error', error, '', module_name))
+                        'VALUES (%s, %s, %s, %s, %s, %s, %s)',
+                        (self._name, 'en_US', 'error', error, '', module_name,
+                            False))
 
     def _get_error_messages(self):
         return self._error_messages.values()
@@ -306,7 +371,7 @@ class Model(object):
             error_description='', error_description_args=None,
             raise_exception=True, context=None):
         '''
-        Raise an exception that will be display as an error message
+        Raise an exception that will be displayed as an error message
         in the client.
 
         :param cursor: the database cursor
@@ -384,8 +449,8 @@ class Model(object):
             warning_args=None, warning_description='',
             warning_description_args=None, context=None):
         '''
-        Raise an exception that will be display as a warning message
-        in the client if the user has not yet by-pass it.
+        Raise an exception that will be displayed as a warning message
+        in the client, if the user has not yet bypassed it.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -420,14 +485,14 @@ class Model(object):
     def default_get(self, cursor, user, fields_names, context=None,
             with_rec_name=True):
         '''
-        Return a dict with the default values for each fields_names.
+        Return a dict with the default values for each field in fields_names.
 
         :param cursor: the database cursor
         :param user: the user id
         :param fields_names: a list of fields names
         :param context: the context
         :param with_rec_name: a boolean to add rec_name value
-        :return: a dictionnary with field name as key
+        :return: a dictionary with field name as key
             and default value as value
         '''
         value = {}
@@ -560,13 +625,13 @@ class Model(object):
 
     def fields_get(self, cursor, user, fields_names=None, context=None):
         """
-        Returns the definition of each field in the object
+        Return the definition of each field in the object.
 
         :param cursor: the database cursor
         :param user: the user id
-        :param fields_names: a list of fields names or None for all fields
+        :param fields_names: a list of field names or None for all fields
         :param context: the context
-        :return: a dictionnary with field name as key and definition as value
+        :return: a dictionary with field name as key and definition as value
         """
         if context is None:
             context = {}
