@@ -4,10 +4,12 @@ from trytond.model import Model
 from trytond.model import fields
 from trytond.model.browse import BrowseRecordList, BrowseRecord, BrowseRecordNull
 from trytond.model.browse import EvalEnvironment
+from trytond.tools import safe_eval
 import datetime
 import time
 from decimal import Decimal
 import logging
+from itertools import chain
 
 OPERATORS = (
     'child_of',
@@ -40,7 +42,7 @@ class ModelStorage(Model):
     :write_uid: A Many2One that points to the user who writed the record.
     :write_date: A Date field for date of last write of the record.
     :rec_name: A Function field that return the rec_name of the record.
-    :__constraints: A list of constraints that each record must respect.
+    :_constraints: A list of constraints that each record must respect.
         Each item of this list is a couple ``('function_name', 'error_keyword')``,
         where ``'function_name'`` is the name of a method of the same class,
         which should return a boolean value (``False`` when the constraint is
@@ -50,9 +52,8 @@ class ModelStorage(Model):
     _rec_name = 'name'
 
     id = fields.Integer('ID', readonly=True)
-    create_uid = fields.Many2One('res.user', 'Create User', required=True,
-            readonly=True)
-    create_date = fields.DateTime('Create Date', required=True, readonly=True)
+    create_uid = fields.Many2One('res.user', 'Create User', readonly=True)
+    create_date = fields.DateTime('Create Date', readonly=True)
     write_uid = fields.Many2One('res.user', 'Write User', readonly=True)
     write_date = fields.DateTime('Write Date', readonly=True)
     rec_name = fields.Function('get_rec_name', type='char', string='Name',
@@ -75,11 +76,11 @@ class ModelStorage(Model):
         self._constraints = []
 
     def default_create_uid(self, cursor, user, context=None):
-        "Default value for uid field"
+        "Default value for uid field."
         return int(user)
 
     def default_create_date(self, cursor, user, context=None):
-        "Default value for create_date field"
+        "Default value for create_date field."
         return datetime.datetime.today()
 
     def default_sequence(self, cursor, user, context=None):
@@ -100,6 +101,20 @@ class ModelStorage(Model):
             return res[0]
         return 0
 
+    def __clean_xxx2many_cache(self, cursor, user, context=None):
+        # Clean cursor cache
+        to_clean = [(model._name, field_name)
+                for model_name, model in self.pool.iterobject(type='model')
+                for field_name, target_name in model._xxx2many_targets
+                if target_name == self._name]
+        for cache in cursor.cache.values():
+            for cache in (cache, cache.get('_language_cache', {}).values()):
+                for model_name, field_name in to_clean:
+                    if model_name in cache:
+                        for model_id in cache[model_name]:
+                            if field_name in cache[model_name][model_id]:
+                                del cache[model_name][model_id][field_name]
+
     def create(self, cursor, user, values, context=None):
         '''
         Create records.
@@ -114,6 +129,7 @@ class ModelStorage(Model):
         model_access_obj = self.pool.get('ir.model.access')
         model_access_obj.check(cursor, user, self._name, 'create',
                 context=context)
+        self.__clean_xxx2many_cache(cursor, user, context=context)
         return False
 
     def read(self, cursor, user, ids, fields_names=None, context=None):
@@ -139,7 +155,7 @@ class ModelStorage(Model):
 
     def write(self, cursor, user, ids, values, context=None):
         '''
-        Write values on records
+        Write values on records.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -160,6 +176,18 @@ class ModelStorage(Model):
             self.raise_user_error(cursor, 'write_xml_record',
                                   error_description='xml_record_desc',
                                   context=context)
+
+        # Clean cursor cache
+        for cache in cursor.cache.values():
+            for cache in (cache, cache.get('_language_cache', {}).values()):
+                if self._name in cache:
+                    if isinstance(ids, (int, long)):
+                        ids = [ids]
+                    for i in ids:
+                        if i in cache[self._name]:
+                            cache[self._name][i] = {}
+        if ids:
+            self.__clean_xxx2many_cache(cursor, user, context=context)
         return False
 
     def delete(self, cursor, user, ids, context=None):
@@ -180,16 +208,28 @@ class ModelStorage(Model):
             self.raise_user_error(cursor, 'delete_xml_record',
                                   error_description='xml_record_desc',
                                   context=context)
+
+        # Clean cursor cache
+        for cache in cursor.cache.values():
+            for cache in (cache, cache.get('_language_cache', {}).values()):
+                if self._name in cache:
+                    if isinstance(ids, (int, long)):
+                        ids = [ids]
+                    for i in ids:
+                        if i in cache[self._name]:
+                            del cache[self._name][i]
+        if ids:
+            self.__clean_xxx2many_cache(cursor, user, context=context)
         return False
 
     def copy(self, cursor, user, ids, default=None, context=None):
         '''
-        Duplicate the record in ids.
+        Duplicate the record(s) in ids.
 
         :param cursor: the database cursor
         :param user: the user id
         :param ids: a list of ids or an id
-        :param default: a dictionnary with field name as keys and
+        :param default: a dictionary with field name as keys and
             new value for the field as value
         :param context: the context
         :return: a list of new ids or the new id
@@ -294,28 +334,28 @@ class ModelStorage(Model):
     def search(self, cursor, user, domain, offset=0, limit=None, order=None,
             context=None, count=False):
         '''
-        Return a list of id that match the clauses defined in args.
+        Return a list of ids that match the clauses defined in args.
 
         :param cursor: the database cursor
         :param user: the user id
         :param domain: a list of tuples or lists
-            lists are construct like this:
+            lists are constructed like this:
             ``['operator', args, args, ...]``
             operator can be 'AND' or 'OR', if it is missing the default
             value will be 'AND'
-            tuples are construct like this:
+            tuples are constructed like this:
             ``('field name', 'operator', value)``
             field name: is a field name from the model or a relational field
             by using '.' as separator.
             operator must be in OPERATORS
-        :param offset: an integer to specify the offset for the result
+        :param offset: an integer to specify the offset of the result
         :param limit: an integer to specify the number of result
-        :param order: a list of tuple that are constructed like this:
+        :param order: a list of tuples that are constructed like this:
             ``('field name', 'DESC|ASC')``
-            it allow to specify the order of result
+            allowing to specify the order of result
         :param context: the context
-        :param count: a boolean to return only the len of the result
-        :return: a list of ids or an interger
+        :param count: a boolean to return only the length of the result
+        :return: a list of ids or an integer
         '''
         if count:
             return 0
@@ -323,7 +363,7 @@ class ModelStorage(Model):
 
     def search_count(self, cursor, user, domain, context=None):
         '''
-        Return the number of record that match the domain. (See search)
+        Return the number of records that match the domain. (See search)
 
         :param cursor: the database cursor
         :param user: the user id
@@ -339,31 +379,58 @@ class ModelStorage(Model):
     def search_read(self, cursor, user, domain, offset=0, limit=None, order=None,
             context=None, fields_names=None):
         '''
-        Call search function and read in once.
-        Usefull for the client to reduce the number of calls.
+        Call search and read functions at once.
+        Useful for the client to reduce the number of calls.
 
         :param cursor: the database cursor
         :param user: the user id
         :param domain: a domain like in search
         :param offset: an integer to specify the offset for the result
-        :param limit: an integer to specify the number of result
-        :param order: a list of tuple that are constructed like this:
+        :param limit: an integer to specify the number of records
+        :param order: a list of tuples that are constructed like this:
             ('field name', 'DESC|ASC')
-            it allow to specify the order of result
+            allowing to to specify the order of result
         :param context: the context
         :param fields_names: fields names to read if None read all fields
-        :return: a list of dictionnary or a dictionnary if limit is 1
-            the dictionnaries will have fields names as key
-            and fields value as value
+        :return: a list of dictionaries or a dictionary if limit is 1
+            the dictionaries will have field names as key
+            and field values as value
         '''
         ids = self.search(cursor, user, domain, offset=offset, limit=limit,
                 order=order, context=context)
+
+        if not fields_names:
+            fields_names = list(set(self._columns.keys() \
+                    + self._inherit_fields.keys()))
+        if 'id' not in fields_names:
+            fields_names.append('id')
+
+        res = []
+        for model in self.browse(cursor, user, ids, context=context):
+            record = {}
+            for fields_name in set(fields_names):
+                fields = fields_name.split('.')
+                while fields:
+                    field_name = fields.pop(0)
+                    if fields_name not in record:
+                        record[fields_name] = model[field_name]
+                    else:
+                        if isinstance(record[fields_name], BrowseRecordNull):
+                            continue
+                        record[fields_name] = record[fields_name][field_name]
+                if isinstance(record[fields_name], BrowseRecordNull):
+                    record[fields_name] = False
+                elif isinstance(record[fields_name], BrowseRecord):
+                    record[fields_name] = record[fields_name].id
+                elif isinstance(record[fields_name], BrowseRecordList):
+                    record[fields_name] = [x.id for x in record[fields_name]]
+            res.append(record)
+
         if limit == 1:
             if not ids:
                 return []
-            ids = ids[0]
-        return self.read(cursor, user, ids, fields_names=fields_names,
-                context=context)
+            return res[0]
+        return res
 
     def _search_domain_active(self, domain, active_test=True, context=None):
         if context is None:
@@ -400,7 +467,7 @@ class ModelStorage(Model):
     def get_rec_name(self, cursor, user, ids, name, arg, context=None):
         '''
         Return a dictionary with id as key and rec_name as value.
-        It is used by the Function field rec_name
+        It is used by the Function field rec_name.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -423,7 +490,7 @@ class ModelStorage(Model):
 
     def search_rec_name(self, cursor, user, name, args, context=None):
         '''
-        Return a list of arguments for search on rec_name
+        Return a list of arguments for search on rec_name.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -441,7 +508,7 @@ class ModelStorage(Model):
 
     def browse(self, cursor, user, ids, context=None):
         '''
-        Return a browse a BrowseRecordList for the ids
+        Return a BrowseRecordList for the ids
             or BrowseRecord if ids is a integer.
 
         :param cursor: the database cursor
@@ -450,12 +517,12 @@ class ModelStorage(Model):
         :param context: the context
         :return: a BrowseRecordList or a BrowseRecord
         '''
-        cache = {}
         if isinstance(ids, (int, long)):
-            return BrowseRecord(cursor, user, ids, self, cache,
-                    context=context)
-        return BrowseRecordList([BrowseRecord(cursor, user, x, self, cache,
-            context=context) for x in ids], context)
+            return BrowseRecord(cursor, user, ids, self, context=context)
+        local_cache = {}
+        return BrowseRecordList([BrowseRecord(cursor, user, x, self,
+            local_cache=local_cache, context=context) for x in ids],
+            context=context)
 
     def __export_row(self, cursor, user, row, fields_names, context=None):
         lines = []
@@ -496,15 +563,15 @@ class ModelStorage(Model):
 
     def export_data(self, cursor, user, ids, fields_names, context=None):
         '''
-        Return list of list of values for each ids.
-        The list of values follow the fields_names.
-        Relational fields are defined with '/' at any deep.
+        Return list of list of values for each id in ids.
+        The list of values follows fields_names.
+        Relational fields are defined with '/' at any depth.
 
         :param cursor: the database cursor
         :param ids: a list of ids
-        :param fields_names: a list of fields names
+        :param fields_names: a list of field names
         :param context: the context
-        :return: a list of list of values for each ids
+        :return: a list of list of values for each id in ids
         '''
         fields_names = [x.split('/') for x in fields_names]
         datas = []
@@ -514,13 +581,13 @@ class ModelStorage(Model):
 
     def import_data(self, cursor, user, fields_names, datas, context=None):
         '''
-        Create record for each values in datas.
-        The fields name of values must be defined in fields_names.
+        Create records for all values in datas.
+        The field names of values must be defined in fields_names.
 
         :param cursor: the database cursor
         :param user: the user id
         :param fields_names: a list of fields names
-        :param datas: the datas to import
+        :param datas: the data to import
         :param context: the context
         :return: a tuple with
             - the number of records imported
@@ -617,6 +684,9 @@ class ModelStorage(Model):
                                 logger.warning(
                                     'Relation not found: ' + line[i] + \
                                         ' on ' + relation + ' !\n')
+                                res = False
+                            else:
+                                res = res[0]
                     elif fields_def[field[len(prefix)]]['type'] == 'many2many':
                         res = []
                         if line[i]:
@@ -713,15 +783,15 @@ class ModelStorage(Model):
         originating from xml data. This is used by write and delete
         functions: if the return value is True the records can be
         written/deleted, False otherwise. The default behaviour is to
-        forbid all modification on records/fields originating from
+        forbid any modification on records/fields originating from
         xml. Values is the dictionary of written values. If values is
         equal to None, no field by field check is performed, False is
-        return has soon has one of the record comes from the xml.
+        returned as soon as one of the record comes from the xml.
 
         :param cursor: the database cursor
         :param user: the user id
         :param ids: a list of ids or an id
-        :param values: a dictionary with fields names as key and
+        :param values: a dictionary with field names as key and
             written values as value
         :param context: the context
         :return: True or False
@@ -744,7 +814,7 @@ class ModelStorage(Model):
                 context=context):
             if not line.values:
                 continue
-            xml_values = eval(line.values, {
+            xml_values = safe_eval(line.values, {
                 'Decimal': Decimal,
                 'datetime': datetime,
                 })
@@ -755,7 +825,7 @@ class ModelStorage(Model):
 
     def check_recursion(self, cursor, user, ids, parent='parent'):
         '''
-        Function that check if there is no recursion in the tree
+        Function that checks if there is no recursion in the tree
         composed with parent as parent field name.
 
         :param cursor: the database cursor
@@ -764,16 +834,40 @@ class ModelStorage(Model):
         :param parent: the parent field name
         :return: True or False
         '''
-        ids_parent = ids[:]
-        while len(ids_parent):
-            ids_parent2 = set()
-            for record in self.browse(cursor, user, ids_parent):
-                if record[parent].id:
-                    ids_parent2.add(record[parent].id)
-            ids_parent = list(ids_parent2)
-            for i in ids_parent:
-                if i in ids:
-                    return False
+        if parent in self._columns:
+            parent_type = self._columns[parent]._type
+        elif parent in self._inherit_fields:
+            parent_type = self._inherit_fields[parent][2]._type
+        else:
+            raise Exception('Field %s not available on object "%s"' % \
+                    (parent, self._name))
+
+        if parent_type not in ('many2one', 'many2many'):
+            raise Exception(
+                    'Unsupported field type "%s" for field "%s" on "%s"' % \
+                    (parent_type, parent, self._name))
+
+        records = self.browse(cursor, user, ids)
+        visited = set()
+
+        for record in records:
+            walked = set()
+            walker = record[parent]
+            while walker:
+                if parent_type == 'many2many':
+                    for w in walker:
+                        walked.add(w.id)
+                        if w.id == record.id:
+                            return False
+                    walker = list(chain(*(w[parent] for w in walker
+                            if w.id not in visited)))
+                else:
+                    walked.add(walker.id)
+                    if walker.id == record.id:
+                        return False
+                    walker = walker[parent] not in visited and walker[parent]
+            visited.update(walked)
+
         return True
 
     def _get_error_args(self, cursor, user, field_name, context=None):
@@ -810,7 +904,8 @@ class ModelStorage(Model):
             if not getattr(self, field[0])(cursor, user, ids):
                 self.raise_user_error(cursor, field[1], context=context)
 
-        if not 'res.user' in self.pool.object_name_list():
+        if not 'res.user' in self.pool.object_name_list() \
+                or user == 0:
             ctx_pref = {
             }
         else:
@@ -818,6 +913,25 @@ class ModelStorage(Model):
             ctx_pref = user_obj.get_preferences(cursor, user,
                 context_only=True, context=context)
 
+        def eval_domain(domain, env=None):
+            res = []
+            for arg in domain:
+                if isinstance(arg, basestring):
+                    if arg in ('AND', 'OR'):
+                        res.append(arg)
+                    else:
+                        if not env:
+                            return True
+                        res.append(safe_eval(arg, env))
+                elif isinstance(arg, tuple):
+                    res.append(arg)
+                elif isinstance(arg, list):
+                    arg = eval_domain(arg, env=env)
+                    if arg == True:
+                        return True
+                    res.append(arg)
+            return res
+
         context.update(ctx_pref)
         records = self.browse(cursor, user, ids, context=context)
         for field_name, field in self._columns.iteritems():
@@ -828,7 +942,7 @@ class ModelStorage(Model):
                     relation_obj = self.pool.get(field.model_name)
                 else:
                     relation_obj = field.get_target(self.pool)
-                if isinstance(field.domain, basestring):
+                if eval_domain(field.domain) == True:
                     ctx = context.copy()
                     ctx.update(ctx_pref)
                     for record in records:
@@ -838,7 +952,7 @@ class ModelStorage(Model):
                         env['time'] = time
                         env['context'] = context
                         env['active_id'] = record.id
-                        domain = eval(field.domain, env)
+                        domain = eval_domain(field.domain, env=env)
                         relation_ids = []
                         if record[field_name]:
                             if field._type in ('many2one',):
@@ -890,7 +1004,7 @@ class ModelStorage(Model):
                         env['time'] = time
                         env['context'] = context
                         env['active_id'] = record.id
-                        required = eval(field.states['required'], env)
+                        required = safe_eval(field.states['required'], env)
                         if required and not record[field_name]:
                             self.raise_user_error(cursor,
                                     'required_validation_record',
@@ -906,6 +1020,25 @@ class ModelStorage(Model):
                                         error_args=self._get_error_args(cursor,
                                             user, field_name, context=context),
                                         context=context)
+            # validate required
+            if field.required:
+                for record in records:
+                    if isinstance(record[field_name], (BrowseRecordNull,
+                        type(None), type(False))) and not record[field_name]:
+                        self.raise_user_error(cursor,
+                                'required_validation_record',
+                                error_args=self._get_error_args(cursor,
+                                    user, field_name, context=context),
+                                context=context)
+            # validate size
+            if hasattr(field, 'size') and field.size:
+                for record in records:
+                    if len(record[field_name] or '') > field.size:
+                        self.raise_user_error(cursor,
+                                'size_validation_record',
+                                error_args=self._get_error_args(cursor,
+                                    user, field_name, context=context),
+                                context=context)
 
     def _clean_defaults(self, defaults):
         vals = {}
@@ -933,7 +1066,7 @@ class ModelStorage(Model):
 
     def workflow_trigger_trigger(self, cursor, user, ids, context=None):
         '''
-        Trigger trigger event
+        Trigger a trigger event.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -950,6 +1083,8 @@ class ModelStorage(Model):
             ('res_id', 'in', ids),
             ('model', '=', self._name),
             ], context=context)
-        for trigger in trigger_obj.browse(cursor, 0, trigger_ids,
-                context=context):
-            instance_obj.update(cursor, user, trigger.instance, context=context)
+        instances = set([trigger.instance \
+                for trigger in trigger_obj.browse(cursor, 0, trigger_ids,
+                    context=context)])
+        for instance in instances:
+            instance_obj.update(cursor, user, instance, context=context)
