@@ -12,18 +12,21 @@ import logging
 
 OPJ = os.path.join
 MODULES_PATH = os.path.dirname(__file__)
-sys.path.insert(1, MODULES_PATH)
 
 MODULES = []
 
 EGG_MODULES = {}
-try:
-    import pkg_resources
-    for ep in pkg_resources.iter_entry_points('trytond.modules'):
-        mod_name = ep.module_name.split('.')[-1]
-        EGG_MODULES[mod_name] = ep
-except ImportError:
-    pass
+
+def update_egg_modules():
+    global EGG_MODULES
+    try:
+        import pkg_resources
+        for ep in pkg_resources.iter_entry_points('trytond.modules'):
+            mod_name = ep.module_name.split('.')[-1]
+            EGG_MODULES[mod_name] = ep
+    except ImportError:
+        pass
+update_egg_modules()
 
 
 class Graph(dict):
@@ -141,7 +144,8 @@ def create_graph(module_list, force=None):
             mod_path = OPJ(ep.dist.location, 'trytond', 'modules', module)
         if os.path.isfile(tryton_file) or zipfile.is_zipfile(mod_path+'.zip'):
             try:
-                info = eval(tools.file_open(tryton_file, subdir='').read())
+                info = tools.safe_eval(tools.file_open(tryton_file,
+                    subdir='').read())
             except:
                 logger.error('%s:eval file %s' % (module, tryton_file))
                 raise
@@ -191,7 +195,7 @@ def load_module_graph(cursor, graph, pool, lang=None):
 
     modules = [x.name for x in graph]
     cursor.execute('SELECT name, state FROM ir_module_module ' \
-            'WHERE name in (' + ','.join(['%s' for x in modules]) + ')',
+            'WHERE name in (' + ','.join(('%s',) * len(modules)) + ')',
             modules)
     module2state = {}
     for name, state in cursor.fetchall():
@@ -304,6 +308,7 @@ def get_module_list():
                 module_list.add(file)
             elif file[-4:] == '.zip':
                 module_list.add(file[:-4])
+    update_egg_modules()
     module_list.update(EGG_MODULES.keys())
     module_list.add('ir')
     module_list.add('workflow')
@@ -312,12 +317,27 @@ def get_module_list():
     module_list.add('tests')
     return list(module_list)
 
-def register_classes():
-    import trytond.ir
-    import trytond.workflow
-    import trytond.res
-    import trytond.webdav
-    import trytond.tests
+def register_classes(reload_p=False):
+    '''
+    Import modules to register the classes in the Pool
+
+    :param reload_p: reload modules instead of import it
+    '''
+    if not reload_p:
+        import trytond.ir
+        import trytond.workflow
+        import trytond.res
+        import trytond.webdav
+        import trytond.tests
+    else:
+        for module in ('trytond.model', 'trytond.report', 'trytond.wizard',
+                'trytond.ir', 'trytond.workflow', 'trytond.res',
+                'trytond.webdav', 'trytond.tests'):
+            for i in sys.modules.keys():
+                if i.startswith(module) \
+                        and i != module:
+                    del sys.modules[i]
+            reload(sys.modules[module])
 
     logger = logging.getLogger('modules')
 
@@ -329,15 +349,25 @@ def register_classes():
             MODULES.append(module)
             continue
 
+        if reload_p and 'trytond.modules.' + module in sys.modules:
+            for i in sys.modules.keys():
+                if i.startswith('trytond.modules.' + module) \
+                        and i != 'trytond.modules.' + module \
+                        and getattr(sys.modules[i], '_TRYTON_RELOAD', True):
+                    del sys.modules[i]
+            reload(sys.modules['trytond.modules.' + module])
+            continue
+
         if os.path.isfile(OPJ(MODULES_PATH, module + '.zip')):
             mod_path = OPJ(MODULES_PATH, module + '.zip')
             zimp = zipimport.zipimporter(mod_path)
-            zimp.load_module(module)
+            zimp.load_module('trytond.modules.' + module)
         elif os.path.isdir(OPJ(MODULES_PATH, module)):
             mod_file, pathname, description = imp.find_module(module,
                     [MODULES_PATH])
             try:
-                imp.load_module(module, mod_file, pathname, description)
+                imp.load_module('trytond.modules.' + module, mod_file,
+                        pathname, description)
             finally:
                 if mod_file is not None:
                     mod_file.close()
@@ -348,7 +378,8 @@ def register_classes():
             mod_file, pathname, description = imp.find_module(module,
                     [mod_path])
             try:
-                imp.load_module(module, mod_file, pathname, description)
+                imp.load_module('trytond.modules.' + module, mod_file,
+                        pathname, description)
             finally:
                 if mod_file is not None:
                     mod_file.close()
@@ -392,8 +423,9 @@ def load_modules(database_name, pool, update=False, lang=None):
         if update:
             cursor.execute("SELECT name FROM ir_module_module " \
                     "WHERE state IN ('to remove')")
-            if cursor.rowcount:
-                for (mod_name,) in cursor.fetchall():
+            fetchall = cursor.fetchall()
+            if fetchall:
+                for (mod_name,) in fetchall:
                     #TODO check if ressource not updated by the user
                     cursor.execute('SELECT model, db_id FROM ir_model_data ' \
                             'WHERE module = %s ' \
