@@ -1,6 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from trytond.modules import load_modules
+from trytond.modules import load_modules, register_classes
 from threading import RLock
 import logging
 
@@ -54,6 +54,43 @@ class Pool(object):
                 break
         Pool.classes[type].setdefault(module, []).append(cls)
 
+    @staticmethod
+    def start():
+        '''
+        Start/restart the Pool
+        '''
+        Pool._lock.acquire()
+        try:
+            reload_p = False
+            for type in Pool.classes:
+                if Pool.classes[type]:
+                    reload_p = True
+                Pool.classes[type] = {}
+            try:
+                register_classes(reload_p=reload_p)
+            except:
+                if not reload_p:
+                    raise
+            for db_name in Pool.database_list():
+                Pool(db_name).init()
+        finally:
+            Pool._lock.release()
+
+    @staticmethod
+    def stop(database_name):
+        '''
+        Stop the Pool
+        '''
+        Pool._lock.acquire()
+        lock = Pool._locks.setdefault(database_name, RLock())
+        Pool._lock.release()
+        lock.acquire()
+        if database_name in Pool._pool:
+            del Pool._pool[database_name]
+        if database_name in Pool._pools:
+            del Pool._pools[database_name]
+        lock.release()
+
     @staticmethod
     def database_list():
         '''
@@ -124,6 +161,15 @@ class Pool(object):
             return res
         return self._pool[self.database_name][type].keys()
 
+    def iterobject(self, type='model'):
+        '''
+        Return an iterator over object name, object
+
+        :param type: the type
+        :return: an iterator
+        '''
+        return self._pool[self.database_name][type].iteritems()
+
     def instanciate(self, module):
         '''
         Instanciate objects for a module
