@@ -1,5 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+_TRYTON_RELOAD = False
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.config import CONFIG
 from trytond.security import login
@@ -17,16 +18,21 @@ import base64
 import time
 from DAV import AuthServer, WebDAVServer, iface
 from DAV.errors import *
-from DAV.constants import COLLECTION, OBJECT
+from DAV.constants import COLLECTION, OBJECT, DAV_VERSION_1, DAV_VERSION_2
 from DAV.utils import get_uriparentpath, get_urifilename, quote_uri
 from DAV.davcmd import copyone, copytree, moveone, movetree, delone, deltree
 import urllib
+import sys
+import traceback
+import logging
+from threading import local
+import xml.dom.minidom
+domimpl = xml.dom.minidom.getDOMImplementation()
+
+DAV_VERSION_1['version'] += ',access-control'
+DAV_VERSION_2['version'] += ',access-control'
 
 # Local int for multi-thread
-import sys
-from threading import local
-
-
 class LocalInt(local):
 
     def __init__(self, value=0):
@@ -37,6 +43,7 @@ class LocalInt(local):
 
 USER_ID = LocalInt(0)
 CACHE = LocalDict()
+DATABASE = LocalDict()
 
 # Fix for bad use of Document in DAV.utils make_xmlresponse
 from DAV.utils import VERSION as DAV_VERSION
@@ -52,6 +59,15 @@ if not hasattr(WebDAVServer.DAVRequestHandler, '_config'):
         lockemulation = False
         verbose = False
 
+        def getboolean(self, name):
+            return bool(self.get(name))
+
+        def get(self, name, default=None):
+            try:
+                return self[name]
+            except:
+                return default
+
 
     class _Config:
         DAV = DAV()
@@ -61,7 +77,7 @@ if not hasattr(WebDAVServer.DAVRequestHandler, '_config'):
 
 class BaseThreadedHTTPServer(SocketServer.ThreadingMixIn,
         BaseHTTPServer.HTTPServer):
-
+    timeout = 1
     max_children = CONFIG['max_thread']
 
     def server_bind(self):
@@ -140,9 +156,22 @@ class TrytonDAVInterface(iface.dav_interface):
         self.baseuri = '%s://%s:%s/' % (protocol, interface or socket.gethostname(), port)
         self.verbose = False
 
-    def _get_dburi(self, uri):
+    def _log_exception(self, exception):
+        if CONFIG['verbose'] or (exception.args \
+                and (str(exception.args[0]) not in \
+                ('NotLogged', 'ConcurrencyException', 'UserError',
+                    'UserWarning')) \
+                    and not isinstance(exception,
+                        (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden))):
+            tb_s = reduce(lambda x, y: x + y,
+                traceback.format_exception(*sys.exc_info()))
+            logger = logging.getLogger('webdav')
+            logger.error('Exception:\n' + tb_s.decode('utf-8', 'ignore'))
+
+    @staticmethod
+    def get_dburi(uri):
         uri = urlparse.urlsplit(uri)[2]
-        if uri[0] == '/':
+        if uri and uri[0] == '/':
             uri = uri[1:]
         dbname, uri = (uri.split('/', 1) + [None])[0:2]
         if dbname:
@@ -151,7 +180,10 @@ class TrytonDAVInterface(iface.dav_interface):
             uri = urllib.unquote_plus(uri)
         return dbname, uri
 
-    def get_childs(self, uri):
+    def _get_dburi(self, uri):
+        return TrytonDAVInterface.get_dburi(uri)
+
+    def get_childs(self, uri, filter=None):
         res = []
         dbname, dburi = self._get_dburi(uri)
         if not dbname:
@@ -165,29 +197,25 @@ class TrytonDAVInterface(iface.dav_interface):
             finally:
                 cursor.close()
             for dbname in lists:
-                res.append(urlparse.urljoin(self.baseuri, dbname))
+                res.append(urlparse.urljoin(uri, dbname))
             return res
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         try:
-            try:
-                collection_obj = pool.get('webdav.collection')
-                if uri[-1:] != '/':
-                    uri += '/'
-                for child in collection_obj.get_childs(cursor, int(USER_ID), dburi,
-                        cache=CACHE):
-                    res.append(urlparse.urljoin(self.baseuri,
-                        uri + child.encode('utf-8')))
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            collection_obj = pool.get('webdav.collection')
+            if uri[-1:] != '/':
+                uri += '/'
+            for child in collection_obj.get_childs(cursor, int(USER_ID), dburi,
+                    filter=filter, cache=CACHE):
+                res.append(uri + child.encode('utf-8'))
+        except KeyError:
+            return res
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def get_data(self, uri):
@@ -218,267 +246,211 @@ class TrytonDAVInterface(iface.dav_interface):
             res += '</body>'
             res += '</html>'
             return res
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_data(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_data(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def put(self, uri, data, content_type=''):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             raise DAV_Forbidden
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.put(cursor, int(USER_ID), dburi, data,
-                        content_type, cache=CACHE)
-                cursor.commit()
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                cursor.rollback()
-                raise
-            except:
-                cursor.rollback()
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.put(cursor, int(USER_ID), dburi, data,
+                    content_type, cache=CACHE)
+            cursor.commit()
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            raise DAV_Error(500)
+        if res:
+            uparts = list(urlparse.urlsplit(uri))
+            uparts[2] = res
+            res = urlparse.urlunsplit(uparts)
         return res
 
     def mkcol(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             raise DAV_Forbidden
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.mkcol(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-                cursor.commit()
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                cursor.rollback()
-                raise
-            except:
-                cursor.rollback()
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.mkcol(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+            cursor.commit()
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            raise DAV_Error(500)
         return res
 
     def _get_dav_resourcetype(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             return COLLECTION
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_resourcetype(cursor, int(USER_ID), dburi,
-                    cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_resourcetype(cursor, int(USER_ID), dburi,
+                cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def _get_dav_displayname(self, uri):
-        raise DAV_Secret
+        return uri.split('/')[-1]
 
     def _get_dav_getcontentlength(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             return '0'
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_contentlength(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_contentlength(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def _get_dav_getcontenttype(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or self.is_collection(uri):
             return "text/html"
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_contenttype(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_contenttype(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
+    def _get_dav_getetag(self, uri):
+        return '"' + str(self.get_lastmodified(uri)) + '"'
+
     def get_creationdate(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             return time.time()
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_creationdate(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_creationdate(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def get_lastmodified(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             return time.time()
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.get_lastmodified(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.get_lastmodified(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def rmcol(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
-            raise DAV_Forbidden
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+            return 403
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.rmcol(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-                cursor.commit()
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                cursor.rollback()
-                raise
-            except:
-                cursor.rollback()
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.rmcol(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+            cursor.commit()
+        except Exception, exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            return 500
         return res
 
     def rm(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
-            raise DAV_Forbidden
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+            return 403
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.rm(cursor, int(USER_ID), dburi,
-                        cache=CACHE)
-                cursor.commit()
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                cursor.rollback()
-                raise
-            except:
-                cursor.rollback()
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.rm(cursor, int(USER_ID), dburi,
+                    cache=CACHE)
+            cursor.commit()
+        except Exception, exception:
+            self._log_exception(exception)
+            cursor.rollback()
+            return 500
         return res
 
     def exists(self, uri):
         dbname, dburi = self._get_dburi(uri)
         if not dbname or not dburi:
             return 1
-        database = Database(dbname)
-        cursor = database.cursor()
-        database_list = Pool.database_list()
-        pool = Pool(dbname)
-        if not dbname in database_list:
-            pool.init()
+        cursor = DATABASE['cursor']
+        pool = Pool(DATABASE['dbname'])
         collection_obj = pool.get('webdav.collection')
         try:
-            try:
-                res = collection_obj.exists(cursor, int(USER_ID), dburi, cache=CACHE)
-            except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden):
-                raise
-            except:
-                raise DAV_Error(500)
-        finally:
-            cursor.close()
+            res = collection_obj.exists(cursor, int(USER_ID), dburi, cache=CACHE)
+        except (DAV_Error, DAV_NotFound, DAV_Secret, DAV_Forbidden), exception:
+            self._log_exception(exception)
+            raise
+        except Exception, exception:
+            self._log_exception(exception)
+            raise DAV_Error(500)
         return res
 
     def is_collection(self, uri):
@@ -513,18 +485,66 @@ class TrytonDAVInterface(iface.dav_interface):
     def copycol(self, src, dst):
         return self.mkcol(dst)
 
+    def _get_dav_current_user_privilege_set(self, uri):
+        doc = domimpl.createDocument(None, 'privilege', None)
+        privilege = doc.documentElement
+        privilege.tagName = 'D:privilege'
+        read = doc.createElement('D:read')
+        privilege.appendChild(read)
+        write = doc.createElement('D:write')
+        privilege.appendChild(write)
+        bind = doc.createElement('D:bind')
+        privilege.appendChild(bind)
+        unbind = doc.createElement('D:unbind')
+        privilege.appendChild(unbind)
+        write_content = doc.createElement('D:write-content')
+        privilege.appendChild(write_content)
+        write_properties = doc.createElement('D:write-properties')
+        privilege.appendChild(write_properties)
+        read_acl = doc.createElement('D:read-acl')
+        privilege.appendChild(read_acl)
+        return privilege
+
+TrytonDAVInterface.PROPS['DAV:'] = tuple(list(TrytonDAVInterface.PROPS['DAV:']) \
+        + ['current-user-privilege-set'])
+
 
 class WebDAVAuthRequestHandler(AuthServer.BufferedAuthRequestHandler,
         WebDAVServer.DAVRequestHandler):
 
+    def finish(self):
+        global DATABASE
+        dbname = None
+        if 'dbname' in DATABASE:
+            dbname = DATABASE['dbname']
+            del DATABASE['dbname']
+        if dbname:
+            Cache.clean(dbname)
+        if 'cursor' in DATABASE:
+            DATABASE['cursor'].close()
+            del DATABASE['cursor']
+        if dbname:
+            Cache.resets(dbname)
+
     def get_userinfo(self, user, password, command=''):
         global USER_ID
+        global DATABASE
+
         dbname = urllib.unquote_plus(self.path.split('/', 2)[1])
+        DATABASE['dbname'] = dbname
         if not dbname:
+            database = Database().connect()
+            DATABASE['cursor'] = database.cursor()
             return 1
-        USER_ID = login(dbname, user, password, cache=False)
-        Cache.clean(dbname)
-        Cache.resets(dbname)
+        USER_ID.value = login(dbname, user, password, cache=False)
+
+        database = Database(dbname).connect()
+        DATABASE['cursor'] = database.cursor()
+        database_list = Pool.database_list()
+        pool = Pool(dbname)
+        if not dbname in database_list:
+            pool.init()
+
         if int(USER_ID):
             return 1
         return 0
