@@ -1,4 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
+from tests import *
+from model import *
 from mptt import *
