@@ -1,4 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 "Convert"
 import re
 try:
@@ -8,6 +9,7 @@ except ImportError:
 import os.path
 from trytond.config import CONFIG
 from trytond.version import VERSION
+from trytond.tools import safe_eval
 import time
 from xml import sax
 from decimal import Decimal
@@ -53,7 +55,7 @@ class MenuitemTagHandler:
                 values[attr] = attributes.get(attr)
 
         if attributes.get('active'):
-            values['active'] = bool(eval(attributes['active']))
+            values['active'] = bool(safe_eval(attributes['active']))
 
         if values.get('parent'):
             values['parent'] = self.mh.get_id(values['parent'])
@@ -63,7 +65,7 @@ class MenuitemTagHandler:
             action_id = self.mh.get_id(values['action'])
 
             # TODO maybe use a prefetch for this:
-            self.mh.cursor.execute(
+            self.mh.cursor.execute(self.mh.cursor.limit_clause(
             "SELECT a.name, a.type, act.view_type, v.type " \
             "FROM ir_action a " \
                 "LEFT JOIN ir_action_report report ON (a.id = report.action) " \
@@ -76,8 +78,8 @@ class MenuitemTagHandler:
                 "OR act.id = %s " \
                 "OR wizard.id = %s " \
                 "OR url.id = %s " \
-            "ORDER by wv.sequence " \
-            "LIMIT 1", (action_id, action_id, action_id, action_id))
+            "ORDER by wv.sequence", 1),
+            (action_id, action_id, action_id, action_id))
             action_name, action_type, view_type, view_mode = \
                 self.mh.cursor.fetchone()
 
@@ -200,7 +202,8 @@ class RecordTagHandler:
                 f_obj = self.mh.pool.get(search_model)
                 answer = f_obj.browse(
                     self.mh.cursor, self.mh.user,
-                    f_obj.search(self.mh.cursor, self.mh.user, eval(search_attr),
+                    f_obj.search(self.mh.cursor, self.mh.user,
+                        safe_eval(search_attr),
                         context={'active_test': False}))
 
                 if not answer: return
@@ -220,7 +223,7 @@ class RecordTagHandler:
                 context['version'] = VERSION.rsplit('.', 1)[0]
                 context['ref'] = self.mh.get_id
                 context['obj'] = lambda *a: 1
-                self.values[field_name] = eval(eval_attr, context)
+                self.values[field_name] = safe_eval(eval_attr, context)
 
         else:
             raise Exception("Tags '%s' not supported inside tag record."% (name,))
@@ -343,16 +346,22 @@ class Fs2bdAccessor:
                 object_ids = ids
         models = model_obj.browse(self.cursor, self.user, object_ids)
         for model in models:
+            if model.id in self.browserecord[module][model_name]:
+                for cache in self.cursor.cache.values():
+                    for cache in (cache, cache.get('_language_cache',
+                        {}).values()):
+                        if model_name in cache \
+                                and model.id in cache[model_name]:
+                            cache[model_name][model.id] = {}
             self.browserecord[module][model_name][model.id] = model
 
     def fetch_new_module(self, module):
-        if module == "ir.ui.menu": raise
         self.fs2db[module] = {}
         module_data_ids = self.modeldata_obj.search(
             self.cursor, self.user, [
                 ('module', '=', module),
                 ('inherit', '=', False),
-                ])
+                ], order=[('db_id', 'ASC')])
 
         record_ids = {}
         for rec in self.modeldata_obj.browse(
@@ -368,12 +377,15 @@ class Fs2bdAccessor:
         for model_name in record_ids.keys():
             model_obj = self.pool.get(model_name)
             self.browserecord[module][model_name] = {}
-            ids = model_obj.search(self.cursor, self.user, [
-                ('id', 'in', record_ids[model_name]),
-                ], context={'active_test': False})
-            models = model_obj.browse(self.cursor, self.user, ids)
-            for model in models:
-                self.browserecord[module][model_name][model.id] = model
+            for i in range(0, len(record_ids[model_name]), self.cursor.IN_MAX):
+                sub_record_ids = record_ids[model_name]\
+                        [i:i + self.cursor.IN_MAX]
+                ids = model_obj.search(self.cursor, self.user, [
+                    ('id', 'in', sub_record_ids),
+                    ], context={'active_test': False})
+                models = model_obj.browse(self.cursor, self.user, ids)
+                for model in models:
+                    self.browserecord[module][model_name][model.id] = model
         self.fetched_modules.append(module)
 
 
@@ -501,7 +513,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 return False
             ref_mode, ref_id = browse_record[key].split(',', 1)
             try:
-                ref_id = eval(ref_id)
+                ref_id = safe_eval(ref_id)
             except:
                 pass
             if isinstance(ref_id, (list, tuple)):
@@ -571,7 +583,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             if not old_values:
                 old_values = {}
             else:
-                old_values = eval(old_values, {
+                old_values = safe_eval(old_values, {
                     'Decimal': Decimal,
                     'datetime': datetime,
                     })
@@ -618,8 +630,9 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                                     'AND module = %s',
                                 (table_name + ',' + field_name,
                                     'en_US', 'model', res_id, module))
-                        if cursor.rowcount:
-                            trans_id = cursor.fetchone()[0]
+                        fetchone = cursor.fetchone()
+                        if fetchone:
+                            trans_id = fetchone[0]
                             cursor.execute('UPDATE ir_translation ' \
                                     'SET src = %s, module = %s ' \
                                     'WHERE id = %s',
@@ -628,10 +641,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                             cursor.execute('INSERT INTO ir_translation ' \
                                     '(name, lang, type, src, res_id, ' \
                                         'value, module, fuzzy) ' \
-                                    'VALUES (%s, %s, %s, %s, %s, %s, %s, false)',
+                                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
                                     (table_name + ',' + field_name,
                                         'en_US', 'model', values[field_name],
-                                        res_id, '', module))
+                                        res_id, '', module, False))
 
                 for table in inherit_db_ids.keys():
                     data_id = self.modeldata_obj.search(cursor, user, [
@@ -759,9 +772,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                                     'AND module = %s',
                                 (table_name + ',' + field_name,
                                     'en_US', 'model', res_id, module))
-                        if cursor.rowcount:
+                        fetchone = cursor.fetchone()
+                        if fetchone:
                             if to_update.get(field_name):
-                                trans_id = cursor.fetchone()[0]
+                                trans_id = fetchone[0]
                                 cursor.execute('UPDATE ir_translation ' \
                                         'SET src = %s, module = %s ' \
                                         'WHERE id = %s',
@@ -770,10 +784,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                             cursor.execute('INSERT INTO ir_translation ' \
                                     '(name, lang, type, src, res_id, ' \
                                         'value, module, fuzzy) ' \
-                                    'VALUES (%s, %s, %s, %s, %s, %s, %s, false)',
+                                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
                                     (table_name + ',' + field_name,
                                         'en_US', 'model', values[field_name],
-                                        res_id, '', module))
+                                        res_id, '', module, False))
 
             if to_update:
                 # re-read it: this ensure that we store the real value
@@ -812,6 +826,8 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         'values': values,
                         'date_update': time.strftime('%Y-%m-%d %H:%M:%S'),
                         })
+            # reset_browsercord to keep cache memory low
+            self.fs2db.reset_browsercord(module, object_ref._name, db_id)
 
         else:
             # this record is new, create it in the db:
@@ -840,11 +856,18 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                             'WHERE name = %s' \
                                 'AND lang = %s ' \
                                 'AND type = %s ' \
-                                'AND res_id = %s',
+                                'AND res_id = %s ' \
+                                'AND module = %s',
                             (table_name + ',' + field_name,
-                                'en_US', 'model', res_id))
-                    if cursor.rowcount:
-                        trans_id = cursor.fetchone()[0]
+                                'en_US', 'model', res_id, module))
+                    trans_id = None
+                    if cursor.rowcount == -1 or cursor.rowcount is None:
+                        data = cursor.fetchone()
+                        if data:
+                            trans_id, = data
+                    elif cursor.rowcount != 0:
+                        trans_id, = cursor.fetchone()
+                    if trans_id:
                         cursor.execute('UPDATE ir_translation ' \
                                 'SET src = %s, module = %s ' \
                                 'WHERE id = %s',
@@ -853,10 +876,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         cursor.execute('INSERT INTO ir_translation ' \
                                 '(name, lang, type, src, res_id, ' \
                                     'value, module, fuzzy) ' \
-                                'VALUES (%s, %s, %s, %s, %s, %s, %s, false)',
+                                'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
                                 (table_name + ',' + field_name,
                                     'en_US', 'model', values[field_name],
-                                    res_id, '', module))
+                                    res_id, '', module, False))
 
             # re-read it: this ensure that we store the real value
             # in the model_data table:
