@@ -7,18 +7,19 @@ import time
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.version import PACKAGE, VERSION, WEBSITE
 from trytond.report import Report
+from trytond.tools import reduce_ids, safe_eval
 
 
 class Collection(ModelSQL, ModelView):
     "Collection"
     _name = "webdav.collection"
     _description = __doc__
-    name = fields.Char('Name', size=128, required=True, select=1)
+    name = fields.Char('Name', required=True, select=1)
     parent = fields.Many2One('webdav.collection', 'Parent',
        ondelete='RESTRICT', domain=[('model', '=', False)])
     childs = fields.One2Many('webdav.collection', 'parent', 'Children')
     model = fields.Many2One('ir.model', 'Model')
-    domain = fields.Char('Domain', size=250)
+    domain = fields.Char('Domain')
 
     def __init__(self):
         super(Collection, self).__init__()
@@ -242,9 +243,12 @@ class Collection(ModelSQL, ModelView):
             cache['_uri2object'][cache_uri] = (object_name, object_id)
         return object_name, object_id
 
-    def get_childs(self, cursor, user, uri, context=None, cache=None):
+    def get_childs(self, cursor, user, uri, filter=None, context=None,
+            cache=None):
         report_obj = self.pool.get('ir.action.report')
         res = []
+        if filter:
+            return []
         if not uri:
             collection_ids = self.search(cursor, user, [
                 ('parent', '=', False),
@@ -267,7 +271,7 @@ class Collection(ModelSQL, ModelView):
                 if not model_obj:
                     return res
                 model_ids = model_obj.search(cursor, user,
-                        eval(collection.domain or "[]"), context=context)
+                        safe_eval(collection.domain or "[]"), context=context)
                 for child in model_obj.browse(cursor, user,
                         model_ids, context=context):
                     if '/' in child.rec_name:
@@ -390,11 +394,11 @@ class Collection(ModelSQL, ModelView):
                 res = None
                 for i in range(0, len(ids), cursor.IN_MAX):
                     sub_ids = ids[i:i + cursor.IN_MAX]
+                    red_sql, red_ids = reduce_ids('id', sub_ids)
                     cursor.execute('SELECT id, ' \
                                 'EXTRACT(epoch FROM create_date) ' \
                             'FROM "' + model_obj._table +'" ' \
-                            'WHERE id IN (' + \
-                                ','.join(['%s' for x in sub_ids]) + ')', sub_ids)
+                            'WHERE ' + red_sql, red_ids)
                     for object_id2, date in cursor.fetchall():
                         if object_id2 == object_id:
                             res = date
@@ -424,12 +428,12 @@ class Collection(ModelSQL, ModelView):
                 res = None
                 for i in range(0, len(ids), cursor.IN_MAX):
                     sub_ids = ids[i:i + cursor.IN_MAX]
+                    red_sql, red_ids = reduce_ids('id', sub_ids)
                     cursor.execute('SELECT id, ' \
                                 'EXTRACT(epoch FROM ' \
                                     'COALESCE(write_date, create_date)) ' \
                             'FROM "' + model_obj._table +'" ' \
-                            'WHERE id IN (' + \
-                                ','.join(['%s' for x in sub_ids]) + ')', sub_ids)
+                            'WHERE ' + red_sql, red_ids)
                     for object_id2, date in cursor.fetchall():
                         if object_id2 == object_id:
                             res = date
@@ -515,7 +519,7 @@ class Collection(ModelSQL, ModelView):
             try:
                 attachment_obj.create(cursor, user, {
                     'name': name,
-                    'datas': base64.encodestring(data),
+                    'datas': base64.encodestring(data or ''),
                     'name': name,
                     'res_model': object_name,
                     'res_id': object_id,
@@ -525,11 +529,11 @@ class Collection(ModelSQL, ModelView):
         else:
             try:
                 attachment_obj.write(cursor, user, object_id2, {
-                    'datas': base64.encodestring(data),
+                    'datas': base64.encodestring(data or ''),
                     }, context=context)
             except:
                 raise DAV_Forbidden
-        return 201
+        return
 
     def mkcol(self, cursor, user, uri, context=None, cache=None):
         from DAV.errors import DAV_Forbidden
