@@ -3,9 +3,15 @@
 {
     'name': 'Workflow',
     'name_de_DE': 'Workflow',
+    'name_es_ES': 'Flujo de trabajo',
+    'name_fr_FR': 'Workflow',
     'description': '''Basic module providing concept and administration of workflows.
 ''',
     'description_de_DE': '''Basismodul für Konzept und Administration von Workflows
+''',
+    'description_es_ES': '''Módulo básico que provee el concepto y administración de flujos de trabajo.
+''',
+    'description_fr_FR': '''Module de base fournissant les concepts et l'administration de workflows.
 ''',
     'active': True,
     'depends': ['ir', 'res'],
